import 'dart:async';

import 'package:explorer/src/data/navigator.dart';
import 'package:explorer/src/ui/provider.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:path/path.dart' as p;
import 'data/models/bread_crumb.dart';
import 'data/models/entry.dart';
import 'data/models/state.dart';

part 'data/controller.dart';

typedef ExplorerBuilder = List<Widget> Function(BuildContext context);

class Explorer extends StatefulWidget {
  const Explorer({
    @required this.controller,
    @required this.builder,
    this.bottomBarBuilder,
    Key key,
  }) : super(key: key);

  final ExplorerController controller;
  final ExplorerBuilder builder;
  final WidgetBuilder bottomBarBuilder;

  @override
  _ExplorerState createState() => _ExplorerState();
}

class _ExplorerState extends State<Explorer>
    with SingleTickerProviderStateMixin {
  ScrollController _scrollController;

  @override
  void initState() {
    widget.controller?._attach(this);
    _scrollController = ScrollController();
    super.initState();
  }

  @override
  void didUpdateWidget(covariant Explorer oldWidget) {
    widget.controller?._attach(this);
    super.didUpdateWidget(oldWidget);
  }

  @override
  void deactivate() {
    widget.controller?._detach();
    super.deactivate();
  }

  @override
  void dispose() {
    widget.controller?._detach();
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => ControllerProvider(
        explorerController: widget.controller,
        scrollController: _scrollController,
        child: Column(
          children: [
            Expanded(
              child: RefreshIndicator(
                onRefresh: widget.controller.refresh,
                child: CustomScrollView(
                  controller: _scrollController,
                  physics: AlwaysScrollableScrollPhysics(),
                  slivers: widget.builder(context),
                ),
              ),
            ),
            if (widget.bottomBarBuilder != null)
              widget.bottomBarBuilder(context),
          ],
        ),
      );
}
